// Copyright 1997 Association for Universities for Research in Astronomy, Inc.,
// Observatory Control System, Gemini Telescopes Project.
// See the file COPYRIGHT for complete details.
//
// $Id: XmlTester.java,v 1.1.1.1 2000/03/28 22:04:02 gillies Exp $
//
import org.w3c.dom.Document;

import java.io.Reader;
import java.io.File;
import java.io.FileReader;

import org.xml.sax.InputSource;
import org.xml.sax.ErrorHandler;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;

class DOMParser implements ErrorHandler 
{

   /** Parser. */
   org.apache.xerces.parsers.DOMParser _parser;

   /**
    * Constructor parses the document using a validating or non-validating
    * parser based on the value of the boolean parameter.
    */
   public DOMParser(boolean validate) 
   {
      _parser = new org.apache.xerces.parsers.DOMParser();
      try {
         _parser.setFeature("http://xml.org/sax/features/validation",
                            validate);
         _parser.setFeature("http://xml.org/sax/features/external-parameter-entities", true);
      } catch (SAXException e) {
         System.err.println("Failed to set validation feature in XML parser");
      }
      _parser.setErrorHandler(this);
   }
    
   /** 
    * Parses the specified URI and returns the document. 
    */
   public Document parse(InputSource src) throws Exception 
   {
      _parser.parse(src);
      return _parser.getDocument();
   }

   /** Warning. */
   public void warning(SAXParseException ex) 
   {
      System.err.println("[Warning Line "+
			 _getLocationString(ex)+"] " +
			 ex.getMessage());
   }

   /** Error. */
   public void error(SAXParseException ex) throws SAXException
   {
      throw new SAXException("[Line "+
			 _getLocationString(ex)+ "] " +
			 ex.getMessage());
   }

   /** Fatal error. */
   public void fatalError(SAXParseException ex) throws SAXException 
   {
      throw new SAXException("[Fatal Error-Line " +
			     _getLocationString(ex)+"] " +
			     ex.getMessage());
   }

   //
   // Private methods
   //
   /** Returns a string of the location. */
   private String _getLocationString(SAXParseException ex) 
   {
      StringBuffer str = new StringBuffer();

      String systemId = ex.getSystemId();
      if (systemId != null) {
	 int index = systemId.lastIndexOf('/');
	 if (index != -1) 
	    systemId = systemId.substring(index + 1);
	 str.append(systemId);
      }
      str.append(':');
      str.append(ex.getLineNumber());
      str.append(':');
      str.append(ex.getColumnNumber());

      return str.toString();
   }
}

/**
* The tester class allows testing of an XML phase1 document against 
* the DTD or with no DTD.
*/
public class XmlTester
{
   // Used for timing parser - before and after times
   private long _before;
   private long _after;


   boolean readDocument(Reader reader, boolean validate)
   {
      System.out.println("Reading an XML Document via DOM");

      _before = System.currentTimeMillis();

      Document d = null;
      DOMParser parser = null;
      try {
	 parser = new DOMParser(validate);

	 InputSource src = new InputSource(reader);

	 // Now do the actual parsing
	 d = parser.parse(src);

      } catch (Exception ex) {
	 System.out.println(ex);
	 return false;
      }
      _after = System.currentTimeMillis();
      printResults("XmlTester parse time", _after - _before);

      return true;
   }

   /** Prints the results. */
   public void printResults(String uri, long time)
   {
      // filename.xml: 631 ms (4 elems, 0 attrs, 78 spaces, 0 chars)
      System.out.print(uri);
      System.out.print(": ");
      System.out.print(time);
      System.out.println(" ms");
   }


   // Use argument dom to test with validating parser or domn to
   // test against non-validating parser.
   static void main(String [] argv)
   {
      if (argv.length != 2) {
	 System.out.println("Usage: XmlTester [dom|domn] file");
	 System.exit(-1);
      }
      boolean validate = argv[0].equals("dom");
      String file = argv[1];
      if (validate) {
	 System.out.println("Parsing and validating file: " + file);
      } else {
	 System.out.println("Parsing file: " + file);
      }

      FileReader fr = null;
      try {
	 fr = new FileReader(file);
      } catch (Exception ex) {
	 System.out.println(ex);
      }

      XmlTester t = new XmlTester();

      if (t.readDocument(fr, validate)) {
	 System.out.println("Success!!!");
      } else {
	 System.out.println("Failure :-(");
      }
   }

}
